@@ -7,7 +7,15 @@
 //----------------------------------------------------------------
 
 namespace caching {
-	emitter::ptr create_restore_emitter(metadata::ptr md, bool clean_shutdown = true);
+
+	enum shutdown_type {
+		CLEAN_SHUTDOWN,
+		NO_CLEAN_SHUTDOWN
+	};
+
+	emitter::ptr create_restore_emitter(metadata::ptr md,
+					    unsigned metadata_version,
+					    shutdown_type st = CLEAN_SHUTDOWN);
 }
 
 //----------------------------------------------------------------
