@@ -1,5 +1,7 @@
 #include "version.h"
 
+#include "base/file_utils.h"
+#include "base/output_file_requirements.h"
 #include "era/commands.h"
 #include "era/metadata.h"
 #include "era/restore_emitter.h"
@@ -32,8 +34,11 @@ namespace {
 	};
 
 	int restore(flags const &fs, bool quiet) {
+		bool metadata_touched = false;
 		try {
 			block_manager<>::ptr bm = open_bm(*fs.output, block_manager<>::READ_WRITE);
+			file_utils::check_file_exists(*fs.output);
+			metadata_touched = true;
 			metadata::ptr md(new metadata(bm, metadata::CREATE));
 			emitter::ptr restorer = create_restore_emitter(*md);
 
@@ -41,6 +46,11 @@ namespace {
 
 		} catch (std::exception &e) {
 			cerr << e.what() << endl;
+			if (metadata_touched)
+				try {
+					file_utils::zero_superblock(*fs.output);
+				} catch (...) {
+				}
 			return 1;
 		}
 
@@ -121,7 +131,10 @@ era_restore_cmd::run(int argc, char **argv)
 		return 1;
 	}
 
-	if (!fs.output) {
+	if (fs.output)
+		check_output_file_requirements(*fs.output);
+
+	else {
 		cerr << "No output file provided." << endl << endl;
 		usage(cerr);
 		return 1;
